@@ -2,6 +2,7 @@
  * lockdown.c
  * com.apple.mobile.lockdownd service implementation.
  *
+ * Copyright (c) 2010 Bryan Forbes All Rights Reserved.
  * Copyright (c) 2008 Zach C. All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -21,6 +22,10 @@
 
 #include <string.h>
 #include <stdlib.h>
+#define _GNU_SOURCE 1
+#define __USE_GNU 1
+#include <stdio.h>
+#include <ctype.h>
 #include <glib.h>
 #include <libtasn1.h>
 #include <gnutls/x509.h>
@@ -49,7 +54,8 @@ const ASN1_ARRAY_TYPE pkcs1_asn1_tab[] = {
  * plist to a previously sent request.
  *
  * @param dict The plist to evaluate.
- * @param query_match Name of the request to match.
+ * @param query_match Name of the request to match or NULL if no match is
+ *        required.
  *
  * @return RESULT_SUCCESS when the result is 'Success',
  *         RESULT_FAILURE when the result is 'Failure',
@@ -71,7 +77,7 @@ static int lockdown_check_result(plist_t dict, const char *query_match)
 		if (!query_value) {
 			return ret;
 		}
-		if (strcmp(query_value, query_match) != 0) {
+		if (query_match && (strcmp(query_value, query_match) != 0)) {
 			free(query_value);
 			return ret;
 		}
@@ -189,8 +195,10 @@ lockdownd_error_t lockdownd_client_free(lockdownd_client_t client)
 		return LOCKDOWN_E_INVALID_ARG;
 	lockdownd_error_t ret = LOCKDOWN_E_UNKNOWN_ERROR;
 
-	if (client->session_id)
+	if (client->session_id) {
 		lockdownd_stop_session(client, client->session_id);
+		free(client->session_id);
+	}
 
 	if (client->parent) {
 		lockdownd_goodbye(client);
@@ -595,6 +603,9 @@ lockdownd_error_t lockdownd_get_device_name(lockdownd_client_t client, char **de
  *
  * @note This function does not pair with the device or start a session. This
  *  has to be done manually by the caller after the client is created.
+ *  The device disconnects automatically if the lockdown connection idles
+ *  for more than 10 seconds. Make sure to call lockdownd_client_free() as soon
+ *  as the connection is no longer needed.
  *
  * @param device The device to create a lockdownd client for
  * @param client The pointer to the location of the new lockdownd_client
@@ -607,8 +618,6 @@ lockdownd_error_t lockdownd_client_new(idevice_t device, lockdownd_client_t *cli
 	if (!client)
 		return LOCKDOWN_E_INVALID_ARG;
 
-	lockdownd_error_t ret = LOCKDOWN_E_SUCCESS;
-
 	property_list_service_client_t plistclient = NULL;
 	if (property_list_service_client_new(device, 0xf27e, &plistclient) != PROPERTY_LIST_SERVICE_E_SUCCESS) {
 		debug_info("could not connect to lockdownd (device %s)", device->uuid);
@@ -620,17 +629,11 @@ lockdownd_error_t lockdownd_client_new(idevice_t device, lockdownd_client_t *cli
 	client_loc->ssl_enabled = 0;
 	client_loc->session_id = NULL;
 	client_loc->uuid = NULL;
-	client_loc->label = NULL;
-	if (label != NULL)
-		client_loc->label = strdup(label);
+	client_loc->label = label ? strdup(label) : NULL;
 
-	if (LOCKDOWN_E_SUCCESS == ret) {
-		*client = client_loc;
-	} else {
-		lockdownd_client_free(client_loc);
-	}
+	*client = client_loc;
 
-	return ret;
+	return LOCKDOWN_E_SUCCESS;
 }
 
 /**
@@ -638,6 +641,10 @@ lockdownd_error_t lockdownd_client_new(idevice_t device, lockdownd_client_t *cli
  * The handshake consists out of query_type, validate_pair, pair and
  * start_session calls. It uses the internal pairing record management.
  *
+ * @note The device disconnects automatically if the lockdown connection idles
+ *  for more than 10 seconds. Make sure to call lockdownd_client_free() as soon
+ *  as the connection is no longer needed.
+ *
  * @param device The device to create a lockdownd client for
  * @param client The pointer to the location of the new lockdownd_client
  * @param label The label to use for communication. Usually the program name.
@@ -656,8 +663,11 @@ lockdownd_error_t lockdownd_client_new_with_handshake(idevice_t device, lockdown
 	char *host_id = NULL;
 	char *type = NULL;
 
-
 	ret = lockdownd_client_new(device, &client_loc, label);
+	if (LOCKDOWN_E_SUCCESS != ret) {
+		debug_info("failed to create lockdownd client.");
+		return ret;
+	}
 
 	/* perform handshake */
 	if (LOCKDOWN_E_SUCCESS != lockdownd_query_type(client_loc, &type)) {
@@ -870,8 +880,16 @@ static lockdownd_error_t lockdownd_do_pair(lockdownd_client_t client, lockdownd_
 	if (ret != LOCKDOWN_E_SUCCESS)
 		return ret;
 
-	if (lockdown_check_result(dict, verb) != RESULT_SUCCESS) {
-		ret = LOCKDOWN_E_PAIRING_FAILED;
+	if (strcmp(verb, "Unpair") == 0) {
+		/* workaround for Unpair giving back ValidatePair,
+		 * seems to be a bug in the device's fw */
+		if (lockdown_check_result(dict, NULL) != RESULT_SUCCESS) {
+			ret = LOCKDOWN_E_PAIRING_FAILED;
+		}
+	} else {
+		if (lockdown_check_result(dict, verb) != RESULT_SUCCESS) {
+			ret = LOCKDOWN_E_PAIRING_FAILED;
+		}
 	}
 
 	/* if pairing succeeded */
@@ -1233,6 +1251,7 @@ lockdownd_error_t lockdownd_start_session(lockdownd_client_t client, const char
 	/* if we have a running session, stop current one first */
 	if (client->session_id) {
 		lockdownd_stop_session(client, client->session_id);
+		free(client->session_id);
 	}
 
 	/* setup request plist */
@@ -1426,7 +1445,7 @@ lockdownd_error_t lockdownd_activate(lockdownd_client_t client, plist_t activati
 	plist_t dict = plist_new_dict();
 	plist_dict_add_label(dict, client->label);
 	plist_dict_insert_item(dict,"Request", plist_new_string("Activate"));
-	plist_dict_insert_item(dict,"ActivationRecord", activation_record);
+	plist_dict_insert_item(dict,"ActivationRecord", plist_copy(activation_record));
 
 	ret = lockdownd_send(client, dict);
 	plist_free(dict);
@@ -1506,3 +1525,100 @@ lockdownd_error_t lockdownd_deactivate(lockdownd_client_t client)
 	return ret;
 }
 
+static void str_remove_spaces(char *source)
+{
+	char *dest = source;
+	while (*source != 0) {
+		if (!isspace(*source)) {
+			*dest++ = *source; /* copy */
+		}
+		source++;
+	}
+	*dest = 0;
+}
+
+/**
+ * Calculates and returns the data classes the device supports from lockdownd.
+ *
+ * @param client An initialized lockdownd client.
+ * @param classes A pointer to store an array of class names. The caller is responsible
+ *  for freeing the memory which can be done using mobilesync_data_classes_free().
+ * @param count The number of items in the classes array.
+ *
+ * @return LOCKDOWN_E_SUCCESS on success,
+ *  LOCKDOWN_E_INVALID_ARG when client is NULL,
+ *  LOCKDOWN_E_NO_RUNNING_SESSION if no session is open, 
+ *  LOCKDOWN_E_PLIST_ERROR if the received plist is broken
+ */
+lockdownd_error_t lockdownd_get_sync_data_classes(lockdownd_client_t client, char ***classes, int *count)
+{
+	if (!client)
+		return LOCKDOWN_E_INVALID_ARG;
+
+	if (!client->session_id)
+		return LOCKDOWN_E_NO_RUNNING_SESSION;
+
+	plist_t dict = NULL;
+	lockdownd_error_t err = LOCKDOWN_E_UNKNOWN_ERROR;
+
+	plist_t value = NULL;
+
+	char **newlist = NULL;
+	char *val = NULL;
+
+	*classes = NULL;
+	*count = 0;
+
+	err = lockdownd_get_value(client, "com.apple.mobile.iTunes", "SyncDataClasses", &dict);
+	if (err != LOCKDOWN_E_SUCCESS) {
+		if (dict) {
+			plist_free(dict);
+		}
+		return err;
+	}
+
+	if (plist_get_node_type(dict) != PLIST_ARRAY) {
+		plist_free(dict);
+		return LOCKDOWN_E_PLIST_ERROR;
+	}
+
+	while((value = plist_array_get_item(dict, *count)) != NULL) {
+			plist_get_string_val(value, &val);
+			newlist = realloc(*classes, sizeof(char*) * (*count+1));
+			str_remove_spaces(val);
+			asprintf(&newlist[*count], "com.apple.%s", val);
+			free(val);
+			val = NULL;
+			*classes = newlist;
+			*count = *count+1;
+	}
+
+	newlist = realloc(*classes, sizeof(char*) * (*count+1));
+	newlist[*count] = NULL;
+	*classes = newlist;
+
+	if (dict) {
+		plist_free(dict);
+	}
+	return LOCKDOWN_E_SUCCESS;
+}
+
+
+/**
+ * Frees memory of an allocated array of data classes as returned by lockdownd_get_sync_data_classes()
+ *
+ * @param classes An array of class names to free.
+ *
+ * @return LOCKDOWN_E_SUCCESS on success
+ */
+lockdownd_error_t lockdownd_data_classes_free(char **classes)
+{
+	if (classes) {
+		int i = 0;
+		while (classes[i++]) {
+			free(classes[i]);
+		}
+		free(classes);
+	}
+	return LOCKDOWN_E_SUCCESS;
+}
